/*
 * LC7461M8100 simulator
 *
 * A. Hojka
 * November 2018
 * */



#include <stm32l0xx_ll_bus.h>
#include <stm32l0xx_ll_exti.h>
#include <stm32l0xx_ll_gpio.h>
#include <stm32l0xx_ll_pwr.h>
#include <stm32l0xx_ll_rcc.h>
#include <stm32l0xx_ll_system.h>
#include <stm32l0xx_ll_utils.h>
#include "stm32l0xx_ll_cortex.h"
#include "stm32l0xx_ll_tim.h"
#include "stm32l0xx.h"
#include "main_utils.h"



/*
 * GPIO config after POR
 * */
void GPIO_cfg(void)
{
	LL_IOP_GRP1_EnableClock(LL_IOP_GRP1_PERIPH_GPIOA);
	LL_IOP_GRP1_EnableClock(LL_IOP_GRP1_PERIPH_GPIOB);

	//Ki_0-Ki_3
	LL_GPIO_SetPinMode(GPIOB, PIN_Ki_0, LL_GPIO_MODE_INPUT);
	LL_GPIO_SetPinMode(GPIOB, PIN_Ki_1, LL_GPIO_MODE_INPUT);
	LL_GPIO_SetPinMode(GPIOB, PIN_Ki_2, LL_GPIO_MODE_INPUT);
	LL_GPIO_SetPinMode(GPIOB, PIN_Ki_3, LL_GPIO_MODE_INPUT);

	LL_GPIO_SetPinPull(GPIOB, PIN_Ki_0, LL_GPIO_PULL_DOWN);
	LL_GPIO_SetPinPull(GPIOB, PIN_Ki_1, LL_GPIO_PULL_DOWN);
	LL_GPIO_SetPinPull(GPIOB, PIN_Ki_2, LL_GPIO_PULL_DOWN);
	LL_GPIO_SetPinPull(GPIOB, PIN_Ki_3, LL_GPIO_PULL_DOWN);

	//Ko_0 to Ko_7
	LL_GPIO_ResetOutputPin(GPIOA,  	PIN_Ko_0 | PIN_Ko_1 | PIN_Ko_2 | PIN_Ko_3 | PIN_Ko_4 | PIN_Ko_5 | PIN_Ko_6 | PIN_Ko_7);

	LL_GPIO_SetPinMode(GPIOA,  		PIN_Ko_0 , LL_GPIO_MODE_OUTPUT);
	LL_GPIO_SetPinMode(GPIOA,  		PIN_Ko_1 , LL_GPIO_MODE_OUTPUT);
	LL_GPIO_SetPinMode(GPIOA,  		PIN_Ko_2 , LL_GPIO_MODE_OUTPUT);
	LL_GPIO_SetPinMode(GPIOA,  		PIN_Ko_3 , LL_GPIO_MODE_OUTPUT);
	LL_GPIO_SetPinMode(GPIOA,  		PIN_Ko_4 , LL_GPIO_MODE_OUTPUT);
	LL_GPIO_SetPinMode(GPIOA,  		PIN_Ko_5 , LL_GPIO_MODE_OUTPUT);
	LL_GPIO_SetPinMode(GPIOA,  		PIN_Ko_6 , LL_GPIO_MODE_OUTPUT);
	LL_GPIO_SetPinMode(GPIOA,  		PIN_Ko_7 , LL_GPIO_MODE_OUTPUT);

	LL_GPIO_SetPinSpeed(GPIOA, 		PIN_Ko_0, LL_GPIO_SPEED_FREQ_LOW);
	LL_GPIO_SetPinSpeed(GPIOA, 		PIN_Ko_1, LL_GPIO_SPEED_FREQ_LOW);
	LL_GPIO_SetPinSpeed(GPIOA, 		PIN_Ko_2, LL_GPIO_SPEED_FREQ_LOW);
	LL_GPIO_SetPinSpeed(GPIOA, 		PIN_Ko_3, LL_GPIO_SPEED_FREQ_LOW);
	LL_GPIO_SetPinSpeed(GPIOA, 		PIN_Ko_4, LL_GPIO_SPEED_FREQ_LOW);
	LL_GPIO_SetPinSpeed(GPIOA, 		PIN_Ko_5, LL_GPIO_SPEED_FREQ_LOW);
	LL_GPIO_SetPinSpeed(GPIOA, 		PIN_Ko_6, LL_GPIO_SPEED_FREQ_LOW);
	LL_GPIO_SetPinSpeed(GPIOA, 		PIN_Ko_7, LL_GPIO_SPEED_FREQ_LOW);

	LL_GPIO_SetPinPull(GPIOA,  		PIN_Ko_0, LL_GPIO_PULL_NO);
	LL_GPIO_SetPinPull(GPIOA,  		PIN_Ko_1, LL_GPIO_PULL_NO);
	LL_GPIO_SetPinPull(GPIOA,  		PIN_Ko_2, LL_GPIO_PULL_NO);
	LL_GPIO_SetPinPull(GPIOA,  		PIN_Ko_3, LL_GPIO_PULL_NO);
	LL_GPIO_SetPinPull(GPIOA,  		PIN_Ko_4, LL_GPIO_PULL_NO);
	LL_GPIO_SetPinPull(GPIOA,  		PIN_Ko_5, LL_GPIO_PULL_NO);
	LL_GPIO_SetPinPull(GPIOA,  		PIN_Ko_6, LL_GPIO_PULL_NO);
	LL_GPIO_SetPinPull(GPIOA,  		PIN_Ko_7, LL_GPIO_PULL_NO);

	LL_GPIO_SetPinOutputType(GPIOA, PIN_Ko_0 | PIN_Ko_1 | PIN_Ko_2 | PIN_Ko_3 | PIN_Ko_4 | PIN_Ko_5 | PIN_Ko_6 | PIN_Ko_7 , LL_GPIO_OUTPUT_PUSHPULL);


	//MOD 		- PA2	(debug)
	LL_GPIO_ResetOutputPin(GPIOA, 	PIN_MOD);
	LL_GPIO_SetPinMode(GPIOA, 		PIN_MOD, LL_GPIO_MODE_OUTPUT);
	LL_GPIO_SetPinSpeed(GPIOA, 		PIN_MOD, LL_GPIO_SPEED_FREQ_MEDIUM);
	LL_GPIO_SetPinPull(GPIOA, 		PIN_MOD, LL_GPIO_PULL_NO);
	LL_GPIO_SetPinOutputType(GPIOA, PIN_MOD, LL_GPIO_OUTPUT_PUSHPULL);

	//OUT		- PA3	(PWM)
	LL_GPIO_ResetOutputPin(GPIOA, 	PIN_OUT);
	LL_GPIO_SetPinMode(GPIOA, 		PIN_OUT, LL_GPIO_MODE_ALTERNATE);
	LL_GPIO_SetAFPin_0_7(GPIOA, 	PIN_OUT, LL_GPIO_AF_0);
	LL_GPIO_SetPinSpeed(GPIOA, 		PIN_OUT, LL_GPIO_SPEED_FREQ_HIGH);
	LL_GPIO_SetPinPull(GPIOA, 		PIN_OUT, LL_GPIO_PULL_NO);
	LL_GPIO_SetPinOutputType(GPIOA, PIN_OUT, LL_GPIO_OUTPUT_PUSHPULL);


	//unused pins-  ANALOG INPUT - by default is analog mode
	//GPIO Port C - never used
}//GPIO_cfg


/* set all output lines to enable wakeup on any keypress
 * switch off GPIO clock
 */
void GPIO_toStop(void)
{
	LL_GPIO_SetPinOutputType(GPIOA, PIN_Ko0toKo7_ALL, LL_GPIO_OUTPUT_PUSHPULL);
	LL_GPIO_SetOutputPin(GPIOA, PIN_Ko0toKo7_ALL);

	LL_IOP_GRP1_DisableClock(LL_IOP_GRP1_PERIPH_GPIOB);
	LL_IOP_GRP1_DisableClock(LL_IOP_GRP1_PERIPH_GPIOA);
}


void GPIO_backFromStop(void)
{
	LL_IOP_GRP1_EnableClock(LL_IOP_GRP1_PERIPH_GPIOA);
	LL_IOP_GRP1_EnableClock(LL_IOP_GRP1_PERIPH_GPIOB);
}




/*
 * setup clock after POR
 *
 * Main clock			 	HSI RC (16MHz) / 4 = 4MHz
 *
 * SYSLCK = HSI16/4			4MHz
 * AHB prescaler			1
 * HCLK 					4MHz
 * FCLK						4MHz
 *
 * SystemTimer				HCLK/8		= 500kHz
 *
 * APB1 peripheral clocks	HCLK /16 	= 250kHz
 * APB1 timer clocks	 	HCLK /16 *2  =500kHz
 *
 * APB2 peripheral clocks	HCLK /2 	= 2MHz
 * APB2 timer clocks	 	HCLK /2 *2  = 4MHz
 */

void CLOCK_cfg(void)
{
	//start on MSI 2MHz
	LL_APB2_GRP1_EnableClock(LL_APB2_GRP1_PERIPH_SYSCFG);

	LL_FLASH_SetLatency(LL_FLASH_LATENCY_0);

	while(LL_FLASH_GetLatency() != LL_FLASH_LATENCY_0){
	}

	LL_RCC_HSI_EnableDivider();								//16MHz div by 4
	LL_RCC_HSI_Enable();

	while(LL_RCC_HSI_IsReady() != 1){
	}

	LL_RCC_HSI_SetCalibTrimming(16);						//std. value

	LL_RCC_SetAHBPrescaler(LL_RCC_SYSCLK_DIV_1);

	LL_RCC_SetAPB1Prescaler(LL_RCC_APB1_DIV_16);
	LL_RCC_SetAPB2Prescaler(LL_RCC_APB2_DIV_2);				//2MHz

	LL_RCC_SetSysClkSource(LL_RCC_SYS_CLKSOURCE_HSI);		//switch to HSI/4

	while(LL_RCC_GetSysClkSource() != LL_RCC_SYS_CLKSOURCE_STATUS_HSI){
	}

	LL_RCC_MSI_Disable();									//switch off unused MSI
	while(LL_RCC_MSI_IsReady() == 1){
	}

	LL_APB2_GRP1_DisableClock(LL_APB2_GRP1_PERIPH_SYSCFG);
}//CLOCK_cfg


/* uC wakes up on MSI clock,
 * switch back to HSI, all previous settings remains
 */
void CLOCK_backFromStop(void)
{
	LL_APB2_GRP1_EnableClock(LL_APB2_GRP1_PERIPH_SYSCFG);

	LL_RCC_HSI_Enable();

	while(LL_RCC_HSI_IsReady() != 1){
	}

	LL_RCC_SetSysClkSource(LL_RCC_SYS_CLKSOURCE_HSI);		//switch to HSI/4

	while(LL_RCC_GetSysClkSource() != LL_RCC_SYS_CLKSOURCE_STATUS_HSI){
	}

	LL_RCC_MSI_Disable();									//switch off unused MSI
	while(LL_RCC_MSI_IsReady() == 1){
	}

	LL_APB2_GRP1_DisableClock(LL_APB2_GRP1_PERIPH_SYSCFG);
}//CLOCK_cfg




/* setup pwr reg. after POR
 * LL_PWR_REGU_VOLTAGE_SCALE3 - 32 kHz to 4.2 MHz (0ws)
 */
void PWR_cfg(void)
{
	LL_APB1_GRP1_EnableClock(LL_APB1_GRP1_PERIPH_PWR);
	LL_PWR_SetRegulVoltageScaling(LL_PWR_REGU_VOLTAGE_SCALE3);
	LL_APB1_GRP1_DisableClock(LL_APB1_GRP1_PERIPH_PWR);
}//PWR_cfg


void PWR_toStop(void)
{
	LL_APB1_GRP1_EnableClock(LL_APB1_GRP1_PERIPH_PWR);

	LL_LPM_DisableEventOnPend();
	LL_LPM_EnableDeepSleep();

	LL_PWR_EnableUltraLowPower();
	LL_PWR_EnableFastWakeUp();

	LL_PWR_ClearFlag_WU();
	LL_PWR_SetPowerMode(LL_PWR_MODE_STOP);

	LL_PWR_SetRegulModeLP(LL_PWR_REGU_LPMODES_LOW_POWER);

	LL_APB1_GRP1_DisableClock(LL_APB1_GRP1_PERIPH_PWR);
}


/*
 * HCLK DIV8 option: SysTick_CTRL_CLKSOURCE_Msk = 0 -> 4MHz/8 = 500kHz
 * 500kHz / 281 -> T = 562us
 *
 * tick pertiod should be 4.5ms/8 = 562.5us
 * error = -0,09%
 */
void SYSTICK_cfg(void)
{
	SysTick->LOAD  = (uint32_t)(280);
	SysTick->VAL   = 0UL;
	//SysTick->CTRL  = SysTick_CTRL_TICKINT_Msk | SysTick_CTRL_ENABLE_Msk;		removed - no isr after POR!!
}


/*
 * disable interrupt and timer
 */
void SYSTICK_toStop(void)
{
	SysTick->CTRL&= ~(SysTick_CTRL_TICKINT_Msk | SysTick_CTRL_ENABLE_Msk);
	SysTick->VAL = 0UL;
}

/*
 * enable interrupt and timer
 */
void SYSTICK_backFromStop(void)
{
	SysTick->CTRL = SysTick_CTRL_TICKINT_Msk | SysTick_CTRL_ENABLE_Msk;
}



/*
 * NVIC POR setup
 */
void NVIC_cfg(void)
{
	NVIC_SetPriority(SVC_IRQn, (1UL << __NVIC_PRIO_BITS) - 1UL);
	NVIC_SetPriority(PendSV_IRQn, (1UL << __NVIC_PRIO_BITS) - 1UL);
	NVIC_SetPriority(SysTick_IRQn, 0);
}



/*
 * prepare event to wakeup from STOP after any keypress

 *	ref.man. page 263
 */
void EXTI_cfg(void)
{
	LL_APB2_GRP1_EnableClock(LL_APB2_GRP1_PERIPH_SYSCFG);

	LL_SYSCFG_SetEXTISource(LL_SYSCFG_EXTI_PORTB, LL_SYSCFG_EXTI_LINE3);	// 	- PIN_Ki_0
	LL_SYSCFG_SetEXTISource(LL_SYSCFG_EXTI_PORTB, LL_SYSCFG_EXTI_LINE4);	//	- PIN_Ki_1
	LL_SYSCFG_SetEXTISource(LL_SYSCFG_EXTI_PORTB, LL_SYSCFG_EXTI_LINE5);	// 	- PIN_Ki_2
	LL_SYSCFG_SetEXTISource(LL_SYSCFG_EXTI_PORTB, LL_SYSCFG_EXTI_LINE6);	// 	- PIN_Ki_3

	LL_EXTI_EnableEvent_0_31(LL_EXTI_LINE_3);
	LL_EXTI_EnableEvent_0_31(LL_EXTI_LINE_4);
	LL_EXTI_EnableEvent_0_31(LL_EXTI_LINE_5);
	LL_EXTI_EnableEvent_0_31(LL_EXTI_LINE_6);

	LL_EXTI_EnableRisingTrig_0_31(LL_EXTI_LINE_3);
	LL_EXTI_EnableRisingTrig_0_31(LL_EXTI_LINE_4);
	LL_EXTI_EnableRisingTrig_0_31(LL_EXTI_LINE_5);
	LL_EXTI_EnableRisingTrig_0_31(LL_EXTI_LINE_6);

	LL_APB2_GRP1_DisableClock(LL_APB2_GRP1_PERIPH_SYSCFG);
}


void EXTI_backFromStop(void)
{
	LL_APB2_GRP1_EnableClock(LL_APB2_GRP1_PERIPH_SYSCFG);

	LL_EXTI_DeInit();
	LL_APB2_GRP1_DisableClock(LL_APB2_GRP1_PERIPH_SYSCFG);
}






/* timer generating IR carrier according to LC7461 datasheet: Th = 8.77us, T=26.3us, D=33,3%
 *
 * fPWM = fAPB2 / 105 = 4MHz / 105 = 38.095kHz
 * error = 0,25%
 * T = 26,25us
 * Tl= 8.75us
 */
void TIM21_cfg(void)
{
LL_TIM_InitTypeDef TIM_InitStruct;
LL_TIM_OC_InitTypeDef TIM_OC_InitStruct;

	  LL_APB2_GRP1_EnableClock(LL_APB2_GRP1_PERIPH_TIM21);

	  TIM_InitStruct.Prescaler = 0;
	  TIM_InitStruct.CounterMode = LL_TIM_COUNTERMODE_UP;
	  TIM_InitStruct.Autoreload = 104;
	  TIM_InitStruct.ClockDivision = LL_TIM_CLOCKDIVISION_DIV1;
	  LL_TIM_Init(TIM21, &TIM_InitStruct);
	  LL_TIM_SetClockSource(TIM21, LL_TIM_CLOCKSOURCE_INTERNAL);
	  LL_TIM_OC_EnablePreload(TIM21, LL_TIM_CHANNEL_CH2);

	  TIM_OC_InitStruct.OCMode = LL_TIM_OCMODE_FORCED_INACTIVE;
	  TIM_OC_InitStruct.OCState = LL_TIM_OCSTATE_ENABLE;
	  TIM_OC_InitStruct.CompareValue = 35;
	  TIM_OC_InitStruct.OCPolarity = LL_TIM_OCPOLARITY_HIGH;
	  LL_TIM_OC_Init(TIM21, LL_TIM_CHANNEL_CH2, &TIM_OC_InitStruct);

	  LL_TIM_OC_DisableFast(TIM21, LL_TIM_CHANNEL_CH2);
	  LL_TIM_SetTriggerOutput(TIM21, LL_TIM_TRGO_RESET);

	  LL_TIM_DisableMasterSlaveMode(TIM21);
	  LL_TIM_EnableCounter(TIM21);
}

/* disable timer making sure OUT is low, disable clock
 */
void TIM21_toStop(void)
{
	LL_TIM_OC_SetMode(TIM21, LL_TIM_CHANNEL_CH2, LL_TIM_OCMODE_FORCED_INACTIVE);
	LL_TIM_DisableCounter(TIM21);
	LL_APB2_GRP1_DisableClock(LL_APB2_GRP1_PERIPH_TIM21);
}

/* enable timer21 after wake up,
 * OC mode remains - LL_TIM_OCMODE_FORCED_INACTIVE
 *
 */
void TIM21_BackFromStop(void)
{
	LL_APB2_GRP1_EnableClock(LL_APB2_GRP1_PERIPH_TIM21);

	TIM21->CNT = 0;
	LL_TIM_EnableCounter(TIM21);
}


/* prepare peripherals before
 * entering deepsleep - STOP no RTC mode
 */
void STOPModeEnter(void)
{
	SYSTICK_toStop();
	TIM21_toStop();
	GPIO_toStop();
	PWR_toStop();
}

/*
 * recover peripherals from STOP
 * */
void STOPModeExit(void)
{
	GPIO_backFromStop();
	CLOCK_backFromStop();
	TIM21_BackFromStop();
	SYSTICK_backFromStop();		//start systick counter and enable exception, key scan begins...
}
